@@ -70,13 +70,15 @@ def _convert(value):
     return do_convert(value)
 
 
-def client_from_dict_config(config, client=None):
+def client_from_dict_config(config, client=None, plugin_param=None):
     """
-    Configure a metlog client, fully configured w/ sender and extensions.
+    Configure a metlog client, fully configured w/ sender and plugins.
 
     :param config: Configuration dictionary.
     :param client: MetlogClient instance to configure. If None, one will be
                    created.
+    :param plugin_param: Configuration dictionary for methods that will be bound
+                         into the MetlogClient instance
 
     The configuration dict supports the following values:
 
@@ -86,11 +88,12 @@ def client_from_dict_config(config, client=None):
       Metlog client default severity value.
     disabled_timers
       Sequence of string tokens identifying timers that are to be deactivated.
+    filters
+      Sequence of 2-tuples `(callable, config)`, where `callable` is dotted
+      notation reference to filter callable and `config` is dictionary config
+      to be passed in during each use of the filter.
     sender
       Nested dictionary containing sender configuration.
-    extensions
-      Nested dictionary specifying dynamic methods to be added to the
-      instantiated Metlog client.
 
     All of the configuration values are optional, but failure to include a
     sender may result in a non-functional Metlog client. Any unrecognized keys
@@ -98,8 +101,7 @@ def client_from_dict_config(config, client=None):
 
     Note that any top level config values starting with `sender_` will be added
     to the `sender` config dictionary, overwriting any values that may already
-    be set. Similarly, any top level config values starting with `extensions_`
-    will be inserted into the `extensions` config dictionary.
+    be set. 
 
     The sender configuration supports the following values:
 
@@ -110,43 +112,69 @@ def client_from_dict_config(config, client=None):
     <kwargs>
       All remaining key-value pairs in the sender config dict will be passed as
       keyword arguments to the sender constructor.
-
-    The extensions dictionary keys should be the attribute name to use for each
-    dynamic method. The values should be the dotted name resolving to the
-    function to be used.
     """
     sender_config = config.get('sender', {})
-    extensions = config.get('extensions', {})
     for key, value in config.items():
         if key.startswith('sender_'):
             sender_config[key[len('sender_'):]] = value
-        elif key.startswith('extensions_'):
-            extensions[key[len('extensions_'):]] = value
     config['sender'] = sender_config
-    config['extensions'] = extensions
 
     logger = config.get('logger', '')
     severity = config.get('severity', 6)
     disabled_timers = config.get('disabled_timers', [])
-
     resolver = DottedNameResolver()
+
+    filters = [(resolver.resolve(filter_dottedname), filter_config) for
+               (filter_dottedname, filter_config) in config.get('filters', [])]
+
     sender_clsname = sender_config.pop('class')
     sender_cls = resolver.resolve(sender_clsname)
     sender_args = sender_config.pop('args', tuple())
     sender = sender_cls(*sender_args, **sender_config)
 
     if client is None:
-        client = MetlogClient(sender, logger, severity, disabled_timers)
+        client = MetlogClient(sender, logger, severity, disabled_timers,
+                              filters)
     else:
-        client.setup(sender, logger, severity, disabled_timers)
+        client.setup(sender, logger, severity, disabled_timers, filters)
 
-    for name, func_name in extensions.items():
-        func = resolver.resolve(func_name)
-        client.add_method(name, func)
+    # Load plugins and pass in config
+    if plugin_param != None:
+        for plugin_name, plugin_config in plugin_param.items():
+            config = plugin_config.pop('plugin.provider')
+            plugin = config(plugin_param[plugin_name])
+            client.add_method(plugin_name, plugin)
 
     return client
 
 
+def _get_filter_config(config, section):
+    """
+    Extract the various filter configuration sections from the config object
+    and return a filters sequence suitable for passing to the client
+    constructor.
+    """
+    return _get_plugin_config(config, section, 'filter')
+
+def _get_plugin_config(config, section, plugin):
+    """
+    Extract the various plugin configuration sections from the config object
+    and return a plugin sequence suitable for passing to the client
+    constructor.
+    """
+    # plugins config
+    plugins_prefix = '%s_%s_' % (section, plugin)
+    plugin_sections = [s for s in config.sections()
+                       if s.startswith(plugins_prefix)]
+    plugins = []
+    for plugin_section in plugin_sections:
+        plugin_config = {}
+        for opt in config.options(plugin_section):
+            plugin_config[opt] = _convert(config.get(plugin_section, opt))
+        plugin_dottedname = plugin_config.pop(plugin)  # 'plugin' key req'd
+        plugins.append((plugin_dottedname, plugin_config))
+    return plugins
+
 def client_from_stream_config(stream, section, client=None):
     """
     Extract configuration data in INI format from a stream object (e.g. a file
@@ -169,7 +197,32 @@ def client_from_stream_config(stream, section, client=None):
     client_dict = {}
     for opt in config.options(section):
         client_dict[opt] = _convert(config.get(section, opt))
-    return client_from_dict_config(client_dict, client)
+
+    filters = _get_filter_config(config, section)
+
+    if filters:
+        client_dict['filters'] = filters
+
+    # Load any plugin configuration
+    plugin_sections = [n for n in config.sections() if n.startswith("%s_plugin" % section)]
+    plugin_param = {}
+
+    resolver = DottedNameResolver()
+
+    for plugin_section in plugin_sections:
+        plugin_name = plugin_section.replace("%s_plugin_" % section, '')
+        plugin_dict = {}
+        for opt in config.options(plugin_section):
+            if opt == 'provider':
+                configurator = resolver.resolve(config.get(plugin_section, opt))
+                plugin_dict['plugin.provider'] = configurator
+                continue
+            plugin_dict[opt] = _convert(config.get(plugin_section, opt))
+        plugin_param[plugin_name] = plugin_dict
+
+    client = client_from_dict_config(client_dict, client, plugin_param)
+
+    return client
 
 
 def client_from_text_config(text, section, client=None):
